/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  Released under the MIT License  - see
  http://www.opensource.org/licenses/mit-license.php

  main.c

  Frequency Counter with LCD display - 

  The input signal, conditioned by a schmidt gate, is presented to
  T0 (PB6). Timer/Counter0 is used in 16 bit mode to count the
  pulses.

  Timer/Counter1 is used to derive a 1 second interval, so the
  frequency can be read from Timer/Counter0 (which is then reset).

  The result is displayed on a standard 2 line 16 character LCD
  display.

  Processor: ATtiny461A
      Clock: External, 8.0 MHz
-----------------------------------------------------------------*/
#define F_CPU 8000000UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay_basic.h>
#include <stdio.h>
#include <string.h>
#include "lcd.h"
#include "main.h"

volatile uint8_t  ucEventFlags=0;         // event flags
volatile uint8_t  ucT0OFCtr=0;            // Timer 0 overflow counter
volatile uint16_t uiT0Ctr=0;

static void system_init(void);
static void display_number( void );

/*-----------------------------------------------------------------
  main()
-----------------------------------------------------------------*/
int main(void)
{
  const char *msg1a = "Frequency Countr";
  const char *msg1b = "Version 1.0.0";
  uint8_t ii;

  system_init();

  // Display the firmware version
  lcd_init( LCD_DISP_ON );
  lcd_home();
  lcd_puts( msg1a );
  lcd_gotoxy( 0, 1 );
  lcd_puts( msg1b );

  for ( ii=0; ii<60; ++ii )
    _delay_loop_2( 0 );  // 32ms
  
  lcd_clrscr();

  /* Replace with your application code */
  while (1) 
  {
    if ( test_bit( ucEventFlags, EVENT_1SEC ) )
    {
      if ( test_bit( PINA, PINA0 ) )
        clear_bit( PORTA, PORTA0 );
      else
        set_bit( PORTA, PORTA0 );

      display_number();

      clear_bit( ucEventFlags, EVENT_1SEC );
    }
  }
}

/*-----------------------------------------------------------------
  system_init()

  PORTB: PB0 (input) MOSI (ISP)
         PB1 (input) MISO (ISP)
         PB2 (input) SCK (ISP)
         PB3 (input) not used
         PB4 (n/a)   XTAL1
         PB5 (n/a)   XTAL2 
         PB6 (input) T0
         PB7 (input) !Reset (ISP)

  PORTA: PA0 (output) Diagnostic LED  
         PA1 (output) LCD RS
         PA2 (output) LCD R/W 
         PA3 (output) LCD E 
         PA4 (output) LCD DB4 
         PA5 (output) LCD DB5
         PA6 (output) LCD DB6 
         PA7 (output) LCD DB7

-----------------------------------------------------------------*/
static void system_init()
{
  DDRB  = 0b00000000;       // PortB: all input
  PORTB = 0b10001111;       // initialize PortB, pullups

  DDRA  = 0b01111111;       // PortA: all output except 7
  PORTA = 0b00000000;       // initialize PortA

  PRR = 0b00000011;         // no ADC or USI

  // Initialize Timer0. Timer0 is used to count T0
  TCCR0A = 0b10000000;       // normal 16-bit counter mode
  TCCR0B = 0b00000111;       // T0 clock source
  set_bit( TIMSK, TOIE0 );	 // enable timer/counter0 overflow interrupt

  // Initialize Timer1. Timer1 is used to create 1 second intervals.
  TCCR1A = 0b00000000;		// normal operation
  TCCR1B = 0b00001111;		// prescaler = 16384 (turns timer1 on)
  TC1H  = 0;
  TCNT1 = INIT_TIMER1;
  set_bit( TIMSK, TOIE1 );	// enable timer/counter1 overflow interrupt

  sei();                    // enable global interrupts
}

/*-----------------------------------------------------------------
-----------------------------------------------------------------*/
static void display_number()
{
  const char *msg3a = "Frequency (Hz)";
  char buffer1[16], buffer2[16];
  unsigned long ulNr;
  char     nn;

  ulNr = ( (unsigned long) ucT0OFCtr << 16 ) | (unsigned long) uiT0Ctr;
  sprintf( buffer1, "%ld", ulNr );
  nn = (char) strlen( buffer1 );

  switch ( nn )
  {
    case 4:
      buffer2[0] = buffer1[0];
      buffer2[1] = ',';
      buffer2[2] = buffer1[1];
      buffer2[3] = buffer1[2];
      buffer2[4] = buffer1[3];
      buffer2[5] = '\0';
      break;
    case 5:
      buffer2[0] = buffer1[0];
      buffer2[1] = buffer1[1];
      buffer2[2] = ',';
      buffer2[3] = buffer1[2];
      buffer2[4] = buffer1[3];
      buffer2[5] = buffer1[4];
      buffer2[6] = '\0';
      break;
    case 6:
      buffer2[0] = buffer1[0];
      buffer2[1] = buffer1[1];
      buffer2[2] = buffer1[2];
      buffer2[3] = ',';
      buffer2[4] = buffer1[3];
      buffer2[5] = buffer1[4];
      buffer2[6] = buffer1[5];
      buffer2[7] = '\0';
      break;
    case 7:
      buffer2[0] = buffer1[0];
      buffer2[1] = ',';
      buffer2[2] = buffer1[1];
      buffer2[3] = buffer1[2];
      buffer2[4] = buffer1[3];
      buffer2[5] = ',';
      buffer2[6] = buffer1[4];
      buffer2[7] = buffer1[5];
      buffer2[8] = buffer1[6];
      buffer2[9] = '\0';
      break;
    case 8:
      buffer2[0] = buffer1[0];
      buffer2[1] = buffer1[1];
      buffer2[2] = ',';
      buffer2[3] = buffer1[2];
      buffer2[4] = buffer1[3];
      buffer2[5] = buffer1[4];
      buffer2[6] = ',';
      buffer2[7] = buffer1[5];
      buffer2[8] = buffer1[6];
      buffer2[9] = buffer1[7];
      buffer2[10] = '\0';
      break;
    default:
      strcpy( buffer2, buffer1 );
  }

  lcd_clrscr();
  lcd_puts( msg3a );
  lcd_gotoxy( 0, 1 );
  lcd_puts( (const char *) buffer2 );
}